//+------------------------------------------------------------------+
//|                                             common_functions.mq4 |
//|                                   Copyright  2010, Bernd Kreuss |
//|                                             Version: 2010.1.27.1 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Bernd Kreuss"

/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <stderror.mqh>

#import "shell32.dll"
int ShellExecuteA(int hWnd, string Verb, string File, string Parameter, string Path, int ShowCommand);
#import

#define SW_SHOWNORMAL 1

/**
* start an external program but DON'T wait for it to finish
*/
void shell(string file, string parameters=""){
   ShellExecuteA(0, "open", file, parameters, NULL, SW_SHOWNORMAL);
}

/**
* use the Comments() display to simulate the behaviour of
* the good old print command, useful for debugging.
* text will be appended as a new line on every call
* and if it has reached 20 lines it will start to scroll.
* if clear is set to True the buffer will be cleared. 
*/
void print(string text, bool clear=False){
   static string print_lines[20];
   static int print_line_position = 0;
   if (IsOptimization()){
      return(0);
   }
   string output="\n";
   string space = "                        ";
   int max_lines = 20;
   int i;
   if (clear){
      for (i=0; i<max_lines; i++){
         print_lines[i] = "";
         print_line_position = 0;
      }
   }
   
   if (print_line_position == max_lines){
      for (i=0; i<max_lines; i++){
         print_lines[i] = print_lines[i+1];
      }
      print_line_position--;
   }
   
   print_lines[print_line_position] = text;
   print_line_position++;
   
   for(i=0; i<print_line_position; i++){
      output = output + print_lines[i] + "\n";
   }
   
   output = stringReplace(output, "\n", "\n" + space);
   Comment(output);
}

/**
* search for the string needle in the string haystack and replace all
* occurrecnes with replace.
*/
string stringReplace(string haystack, string needle, string replace=""){
   string left, right;
   int start=0;
   int rlen = StringLen(replace);
   int nlen = StringLen(needle);
   while (start > -1){
      start = StringFind(haystack, needle, start);
      if (start > -1){
         if(start > 0){
            left = StringSubstr(haystack, 0, start);
         }else{
            left="";
         }
         right = StringSubstr(haystack, start + nlen);
         haystack = left + replace + right;
         start = start + rlen;
      }
   }
   return (haystack);  
}

/**
* create an integer number from a string. 
* Use this like makeMagicNumber(name + Symbol() + Period())
* to automatically create a unique magic number for each
* symbol and timeframe the expert is used on (name should be
* unique to the expert). The algorithm is very rudimentary
* but good enough for this purpose, collisions should not 
* occur very often.
*/
int makeMagicNumber(string str){
   int i,s;
   for (i=0; i<StringLen(str); i++){
      s = s + StringGetChar(str,i) * i * 32;
   }
   return(s);
}

/**
* place a market buy with stop loss, target, magic and Comment
* keeps trying in an infinite loop until the position is open.
*/
int buy(double lots, double sl, double tp, int magic=42, string comment=""){
   int err;
   int ticket;
   while(True){
      RefreshRates();
      Print("Trying to buy (" + lots + "," + Ask + "," + sl + "," + tp + ")");
      ticket = OrderSendN(Symbol(), OP_BUY, lots, Ask, 100, sl, tp, comment, magic, 0, Blue);
      err = GetLastError();
      Print("last Error: " + err);
      if (err != 0){
         if (isTemporaryError(err)){
            Sleep(MathRand()/10);
         }else{
            return(-1);
         }
      }else{
         return(ticket);
      }
   }   
}

/**
* place a market sell with stop loss, target, magic and comment
* keeps trying in an infinite loop until the position is open.
*/
int sell(double lots, double sl, double tp, int magic=42, string comment=""){
   int err;
   int ticket;
   while(True){
      RefreshRates();
      Print("Trying to sell (" + lots + "," + Bid + "," + sl + "," + tp + ")");
      ticket = OrderSendN(Symbol(), OP_SELL, lots, Bid, 100, sl, tp, comment, magic, 0, Red);
      err = GetLastError();
      Print("last Error: " + err);
      if (err != 0){
         if (isTemporaryError(err)){
            Sleep(MathRand()/10);
         }else{
            return(-1);
         }
      }else{
         return(ticket);
      }
   }   
}

/**
* place a buy limit order
*/
int buyLimit(double lots, double price, double sl, double tp, int magic=42, string comment=""){
   return(OrderSendN(Symbol(), OP_BUYLIMIT, lots, price, 1, sl, tp, comment, magic, 0, Navy));
}

/**
* place a sell limit order
*/
int sellLimit(double lots, double price, double sl, double tp, int magic=42, string comment=""){
   return(OrderSendN(Symbol(), OP_SELLLIMIT, lots, price, 1, sl, tp, comment, magic, 0, Maroon));
}

/**
* place a buy stop order
*/
int buyStop(double lots, double price, double sl, double tp, int magic=42, string comment=""){
   return(OrderSendN(Symbol(), OP_BUYSTOP, lots, price, 1, sl, tp, comment, magic, 0, Navy));
}

/**
* place a sell stop order
*/
int sellStop(double lots, double price, double sl, double tp, int magic=42, string comment=""){
   return(OrderSendN(Symbol(), OP_SELLSTOP, lots, price, 1, sl, tp, comment, magic, 0, Maroon));
}



/**
* calculate unrealized P&L, belonging to all open trades with this magic number
*/
double getProfit(int magic){
   int cnt;
   double profit = 0;
   int total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == magic){
         profit += OrderProfit() + OrderSwap();
      }
   }
   return (profit);
}

/**
* calculate realized P&L resulting from all closed trades with this magic number
*/
double getProfitRealized(int magic){
   int cnt;
   double profit = 0;
   int total=OrdersHistoryTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_HISTORY);
      if(OrderMagicNumber() == magic){
         profit += OrderProfit() + OrderSwap();
      }
   }
   return (profit);
}

/**
* get the number of currently open trades of specified type
*/
int getNumOpenOrders(int type, int magic){
   int cnt;
   int num = 0;
   int total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == magic && (type==-1 || OrderType() == type)){
         num++;
      }
   }
   return (num);
}

bool haveOpenTrades(int magic){
   return(getNumOpenOrders(OP_SELL, magic) + getNumOpenOrders(OP_BUY, magic) > 0);
}

int getOpenTradeType(int magic){
   if(getNumOpenOrders(OP_BUY, magic) != 0){
      return (OP_BUY);
   }
   if(getNumOpenOrders(OP_SELL, magic) != 0){
      return (OP_SELL);
   }
   return (-1);
}

/**
* trailTargets()
* will loop through all matching open positions and trail the target 
* towards price if the price moves AGAINST the trade direction (away 
* from the target). This may even trail the target into negative territory.
* 
* If trailtarget is 0 then nothing will be done, a positive number
* means the maximum distance between price and target.
* 
* If max_negative is set to -1 (default) the there is no limitation
* how far into negative territory the target can be trailed, if  
* max_negative is set to 0 then it will only trail down to break even
* and then stop at this point, if it is set to a positive number
* then it will use this number as a limit of how far into loss the
* target may be trailed.
*/
void trailTargets(double trailtarget, int magic, double max_negative=-1){
   int total, cnt, type;
   double op, tp, old_tp, dist;
   bool change;
   
   if (trailtarget != 0){
      total=OrdersTotal();
      for(cnt=total;cnt>=0;cnt--){
         OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber() == magic){
            op = OrderOpenPrice();
            old_tp = OrderTakeProfit();
            type = OrderType();
            change = False;
            
            if (type == OP_BUY){
               tp = Bid + trailtarget;
               dist = tp - op;
               if (max_negative >= 0 && -dist > max_negative){
                  tp = op - max_negative;
               }
               if (tp < old_tp){
                  change = True;
               }
            }
            
            if (type == OP_SELL){
               tp = Ask - trailtarget;
               dist = op - tp;
               if (max_negative >= 0 && -dist > max_negative){
                  tp = op + max_negative;
               }
               if (tp > old_tp){
                  change = True;
               }
            }
            
            if (change){
               OrderModifyN(
                  OrderTicket(),
                  op,
                  OrderStopLoss(),
                  tp,
                  OrderExpiration()
               );
            }
         }
      }
   }
}

/**
* trailStops()
* will loop through all matching open positions and trail their stops
*
* if trailstops is 0 then nothing will be done, a positive number is
* the maximum distance between price and stop, the distance at which
* the stop is trailed behind price.
*
* trailstop_slow is a factor that will increase trailing distance
* when the profit grows. default 1 is normal trailing stop behaviour
*/
void trailStops(double trailstop, int magic, double trailstop_slow=1){
   int total, cnt, type;
   double op, sl, old_sl;
   bool change;
   
   if (trailstop != 0){
      total=OrdersTotal();
      for(cnt=total;cnt>=0;cnt--){
         OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber() == magic){
            op = OrderOpenPrice();
            old_sl = OrderStopLoss();
            type = OrderType();
            change = False;
            
            if (type == OP_BUY){
               sl = op + (Bid - trailstop - op) / trailstop_slow;
               if (sl > op && sl > old_sl){
                  change = True;
               }
            }
            
            if (type == OP_SELL){
               sl = op - (op - Ask - trailstop) / trailstop_slow;
               if (sl < op && (sl < old_sl || old_sl == 0)){
                  change = True;
               }
            }
            
            if (change){
               OrderModifyN(
                  OrderTicket(),
                  op,
                  sl,
                  OrderTakeProfit(),
                  OrderExpiration()
               );
            }
         }
      }
   }
}

void lockProfit(double min_profit, int magic, double distance=0){
   int total, cnt, type;
   double op, sl, old_sl;
   bool change;
   
   total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == magic){
         old_sl = OrderStopLoss();
         op = OrderOpenPrice();
         type = OrderType();
         change = False;
         
         if (type==OP_BUY){
            sl = op + min_profit;
            if (Bid > sl + distance && sl > old_sl){
                  change = True;
            }
         }
         
         if (type==OP_SELL){
            sl = op - min_profit;
            if (Ask < sl - distance && (sl < old_sl || old_sl == 0)){
                  change = True;
            }
         }
         
         if (change){
            OrderModifyN(
               OrderTicket(),
               op,
               sl,
               OrderTakeProfit(),
               OrderExpiration()
            );
         }
      }
   }   
}

bool isOrder(int type, double price, int magic){
   int cnt, num;
   int total=OrdersTotal();
   for(cnt=total-1;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == magic && OrderType() == type){
         if(NormalizeDouble(OrderOpenPrice(), Digits) == NormalizeDouble(price, Digits)){
            num++;
         }
      }
   }
   return (num>0);
}

/** 
* will close all open orders or positions of specified type with our magic number
* this function won't return until all positions are closed
*/
void closeOpenOrders(int type, int magic){
   int total, cnt;
   double price;
   
   
   while (getNumOpenOrders(type, magic) > 0){
      while (IsTradeContextBusy()){
         Print("Waiting for trade context... (OrderClose)");
         Sleep(MathRand()/10);
      }
      total=OrdersTotal();
      Print("Trying to close trades");
      RefreshRates();
      if (type == OP_BUY){
         price = Bid;
      }else{
         price = Ask;
      }
      for(cnt=total;cnt>=0;cnt--){
         OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
         if((type==-1 || OrderType()==type) && OrderMagicNumber() == magic){
            if(IsTradeContextBusy()){
               break; // something else is trading too, back to the while loop.
            }
            if (type == OP_BUYSTOP || type == OP_SELLSTOP || type == OP_BUYLIMIT || type == OP_SELLLIMIT){
               OrderDelete(OrderTicket());
            }else{
               OrderClose(OrderTicket(),OrderLots(),price,999,Yellow);
            }
            Print("last error: " + GetLastError());
         } 
      }
      Sleep(MathRand()/100);
   }
}

/**
* scale out of a position the specified amounts of lots and return
* true if successful.
* This function may change in the future since there is still room
* left for making it more robust. The current implementation will
* search for an open position equal or greater than the required
* lot size and then close parts or all of this position. It will
* only search for *one* position, it is not yet able to close for
* example two one lot postitions if you want to scale out two lots,
* if it doesn't find one single position big enough it will do nothing
* and return False, so you must check the return value and act accordingly.
*/
bool reducePosition(int type, double lots, int magic){
   int i;
   int total_orders = OrdersTotal();
   double price;
   int clr;
   if (type == OP_BUY){
      price = Bid;
      clr = Red;
   }
   if (type == OP_SELL){
      price = Ask;
      clr = Blue;
   }
   for (i=0; i<total_orders; i++){
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber() == magic){
         if (OrderType() == type){
            if (NormalizeDouble (OrderLots() - lots, 2) >= 0){
               OrderClose(OrderTicket(), lots, price, 100, clr);
               if (GetLastError() == 0){
                  return(True);
               }
            }
         }
      }
   }
   return(False);
}

void moveStop(int type, int magic, double stoploss=-1){
   int total, cnt;
   total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==type && OrderMagicNumber() == magic){
         if (stoploss == -1){
            stoploss = OrderOpenPrice();
         }
         if (!isEqualPrice(stoploss, OrderStopLoss())){
            OrderModifyN(
               OrderTicket(),
               OrderOpenPrice(),
               stoploss,
               OrderTakeProfit(),
               OrderExpiration()
            );
         }
      } 
   }
}

void moveOrder(int type, int magic, double price){
   int total, cnt;
   double d, sl, tp;
   total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==type && OrderMagicNumber() == magic){
         if(!isEqualPrice(d, OrderOpenPrice())){
            d = price - OrderOpenPrice();
            if (OrderStopLoss() == 0){
               sl = 0;
            }else{
               sl = OrderStopLoss() + d;
            }
            if (OrderTakeProfit() == 0){
               tp = 0;
            }else{
               tp = OrderTakeProfit() + d;
            }
            OrderModifyN(
               OrderTicket(),
               price,
               sl,
               tp,
               OrderExpiration()
            );
         }
      }
   }
}

void moveTarget(int type, int magic, double target){
   int total, cnt;
   total=OrdersTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==type && OrderMagicNumber() == magic){
         if (!isEqualPrice(target, OrderTakeProfit())){
            OrderModifyN(
               OrderTicket(),
               OrderOpenPrice(),
               OrderStopLoss(),
               target,
               OrderExpiration()
            );
         }
      } 
   }
}

double getLotsOnTable(int magic){
   double total_lots = 0;
   int i;
   int total_orders = OrdersTotal();
   for (i=0; i<total_orders; i++){
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber() == magic){
         if(OrderType() == OP_BUY){
            total_lots += OrderLots();
         }
         if(OrderType() == OP_SELL){
            total_lots -= OrderLots();
         }
      }
   }
   total_lots = MathAbs(total_lots);
   return(total_lots);
}

double getAveragePositionPrice(int magic){
   double total_lots = getLotsOnTable(magic);
   double average_price = 0;
   int i;
   int total_orders = OrdersTotal();
   for (i=0; i<total_orders; i++){
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if (OrderMagicNumber() == magic){
         average_price += OrderOpenPrice()*OrderLots()/total_lots;
      }
   }
   return(average_price);
}

double getLastProfit(int magic){
   int total, cnt;
   total=OrdersHistoryTotal();
   for(cnt=total;cnt>=0;cnt--){
      OrderSelect(cnt, SELECT_BY_POS, MODE_HISTORY);
      if(OrderMagicNumber() == magic){
         return(OrderProfit());
      }
   }
}

string line(string name, datetime t1, double p1, datetime t2, double p2, color clr=Red, string label="", bool ray=False){
   if (!IsOptimization()){
      if (name==""){
         name = "line_" + Time[0];
      }
      if (ObjectFind(name)==-1){
         ObjectCreate(name, OBJ_TREND,0,t1,p1,t2,p2);
      }
      ObjectSet(name, OBJPROP_RAY, ray);
      ObjectSet(name, OBJPROP_COLOR, clr);
      ObjectSet(name, OBJPROP_TIME1, t1);
      ObjectSet(name, OBJPROP_TIME2, t2);
      ObjectSet(name, OBJPROP_PRICE1, p1);
      ObjectSet(name, OBJPROP_PRICE2, p2);
      ObjectSetText(name, label);
   }
   return(name);
}

string horizLine(string name, double price, color clr=Red, string label=""){
   if (!IsOptimization()){
      if (name==""){
         name = "line_" + Time[0];
      }
      if (ObjectFind(name)==-1){
         ObjectCreate(name, OBJ_HLINE, 0, 0, price);
      }else{
         ObjectSet(name, OBJPROP_PRICE1, price);
      }
      ObjectSet(name, OBJPROP_COLOR, clr);
      ObjectSetText(name, label);
   }
   return(name);
}

string vertLine(string name, datetime time, color clr=Red, string label=""){
   if (!IsOptimization()){
      if (name==""){
         name = "line_" + Time[0];
      }
      if (ObjectFind(name)==-1){
         ObjectCreate(name, OBJ_VLINE, 0, time, 0);
      }else{
         ObjectSet(name, OBJPROP_TIME1, time);
      }
      ObjectSet(name, OBJPROP_COLOR, clr);
      ObjectSetText(name, label);
   }
   return(name);
}

string text(string name, string text, datetime time=0, double price=0, color clr=Red, int size=8){
   if (time == 0){
      time = Time[0];
   }
   if (price == 0){
      price = Close[iBarShift(NULL, 0, time)];
   }
   if (name == ""){
      name = "text_" + time;
   }
   if (ObjectFind(name)==-1){
      ObjectCreate(name, OBJ_TEXT, 0, time, price);
   }
   ObjectSet(name, OBJPROP_TIME1, time);
   ObjectSet(name, OBJPROP_PRICE1, price);
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_FONTSIZE, size);
   ObjectSetText(name, text);
   return(name);
}

string label(string name, int x, int y, int corner, string label, color clr=Gray){
   if (!IsOptimization()){
      if (name==""){
         name = "label_" + Time[0];
      }   
      if (!ObjectFind(name)==-1){
         ObjectCreate("c", OBJ_LABEL, 0,0,0);
      }
      ObjectSet(name, OBJPROP_COLOR, clr);
      ObjectSet(name, OBJPROP_CORNER, corner);
      ObjectSet(name, OBJPROP_XDISTANCE, x);
      ObjectSet(name, OBJPROP_YDISTANCE, y);   
      ObjectSetText(name, label);
   }
   return(name);
}

double getLastHigh(int bars){
   int i;
   double value;
   for (i=3; i<100; i++){
      value = iFractals(NULL,0,MODE_UPPER,i);
      if (value!=0){
         return(value);
      }
   }
}

double getLastLow(int bars){
   int i;
   double value;
   for (i=3; i<100; i++){
      value = iFractals(NULL,0,MODE_LOWER,i);
      if (value!=0){
         return(value);
      }
   }
}

double getLotSize(double stop, double risk, double minimum){
   if(stop == 0){
      return(0);
   }
   double value = AccountBalance()*risk;
   double size = value / (stop * 100000);
   size = MathRound(size / minimum) * minimum;
   if (size==0){
      size = minimum;
   }
   return(size);
}

int higherPeriod(int p){
   switch(p){
      case 1: return(5);
      case 5: return(15);
      case 15: return(30);
      case 30: return(60);
      case 60: return(240);
      case 240: return(1440);
      case 1440: return(10080);
      case 10080: return(43200);
      default: return(-1);
   }   
}

void writeProfitCurve(string filename, int magic){
   int i;
   double sum;
   int f = FileOpen(filename, FILE_WRITE);
   FileWrite(f, "time,profit");
   int num = OrdersHistoryTotal();
   for (i=0; i<=num; i++){
      OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
      if (OrderMagicNumber() == magic){
         sum = sum + OrderProfit();
         FileWrite(f, TimeToStr(OrderCloseTime()) + "," + sum);
      }
   }
   FileClose(f);
}

/**
* OrderSend() with automatic normalizing of prices
*/
int OrderSendN(
   string symbol, 
   int cmd, 
   double volume, 
   double price, 
   int slippage, 
   double stoploss,
   double takeprofit,
   string comment="",
   int magic=0,
   datetime expiration=0,
   color arrow_color=CLR_NONE
){
   return(OrderSend(
      symbol,
      cmd, 
      volume, 
      NormalizeDouble(price, Digits), 
      slippage, 
      NormalizeDouble(stoploss, Digits), 
      NormalizeDouble(takeprofit, Digits),
      comment, 
      magic, 
      expiration, 
      arrow_color
   ));
}

/**
* OrderModify() with automatic normalizing of prices
*/
int OrderModifyN(
   int ticket,
   double price,
   double stoploss,
   double takeprofit,
   datetime expiration
){
   bool r;
   int err;
   int retry_time;
   color clr;
   int t = OrderType();
   if (t == OP_BUYLIMIT || t == OP_BUYSTOP){
      clr = Navy;
   }
   if (t == OP_SELLLIMIT || t == OP_SELLSTOP){
      clr = Maroon;
   }
   while (True){
      while (IsTradeContextBusy()){
         Print("Waiting for trade context... (OrderModify)");
         Sleep(MathRand()/10);
      }
      r = OrderModify(
          ticket,
          NormalizeDouble(price, Digits),
          NormalizeDouble(stoploss, Digits),
          NormalizeDouble(takeprofit, Digits),
          expiration,
          clr);
      err = GetLastError();
      if (err == 0 || !isTemporaryError(err)){
         return(r); // exit the loop and the function
      }
   }
}

bool isEqualPrice(double a, double b){
   return(NormalizeDouble(a, Digits) == NormalizeDouble(b, Digits));
}

bool isTemporaryError(int error){
   return(
      error == ERR_COMMON_ERROR ||
      error == ERR_SERVER_BUSY ||
      error == ERR_NO_CONNECTION ||
      error == ERR_MARKET_CLOSED ||
      error == ERR_PRICE_CHANGED ||
      error == ERR_OFF_QUOTES ||
      error == ERR_BROKER_BUSY ||
      error == ERR_REQUOTE ||
      error == ERR_TRADE_CONTEXT_BUSY
    );
}