library pattern; {version: 2010.1.30.1}

{   Copyright (c) 2010 Bernd Kreuss

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program attached at the end of this source code file.
    If not, see <http://www.gnu.org/licenses/>.}

{$mode objfpc}
{$H+}
{$calling stdcall}

type
  {The following data structures are a very MT4 specific
  thing. Many thanks go to chipdude from the mql4.com forum
  for reverse engineering this part of the puzzle.
  The array itself is actually never instantiated by this dll,
  all we use is a pointer to such an array given to us
  by the calling mql4 program so we can peek around in
  metatrader's memory.}
  TCandle = packed record
    time: integer;
    Open,
    High,
    Low,
    Close,
    Volume: double;
  end;
  THistory = array[0..65000] of TCandle;
  PHistory = ^THistory;

  {in mql4 this is a double foo[2], used to pass a pair
  of doubles by reference from MT4 to an imported function}
  TTuple = array[0..1] of double;

const
  pattern_x_range = 100;
  pattern_x_div = 10;
  scan_min_width = 10;

var
  history: PHistory; {all bars in the chart. first bar is oldest one!}
  Bars: integer; {total number of available bars in history}
  size_cache_high: double;
  size_cache_low: double;
  last_error: double;
  price_high: double;
  price_low: double;

  {parameters for the pattern function}
  param1: double;
  param2: double;
  param3: double;

  { This is the pattern function that describes the
  shape of the pattern we want to scan for. Values for
  x range from 0..100, the function return value has no
  restriction for its range, it will be scaled automatically.}
  function pattern(x: double): double; inline;
  begin
    x := x - param1;
    pattern := sqrt(abs(x));
  end;

  { Calculate the range of close prices and return the height
  of this range. The absolute values of min and max will be
  written to global variables for use by the scaling algorithm.}
  function calcPriceRange(Width, LastBar: integer): double;
  var
    i: integer;
    i0: integer;
    i1: integer;
    min: double;
    max: double;

  begin
    i1 := Bars - LastBar - 1;
    i0 := i1 - abs(Width);
    min := history^[i0].Close;
    max := min;
    for i := i0 to i1 do
    begin
      if history^[i].Close < min then min := history^[i].Close;
      if history^[i].Close > max then max := history^[i].Close;
    end;
    price_low := min;
    price_high := max;
    calcPriceRange := max - min;
  end;

  { Determine the y size of our pattern function. The return value is
  cached until the end of the program or until explicitely invalidated.
  The result is written to the tuple that is passed by reference, the
  first entry is the absolute low and the second one the absolute high.}
  procedure patternSize(var size: TTuple; invalidate_cache: boolean);
  var
    i: integer;
    x, y, min, max: double;

  begin
    if (size_cache_high - size_cache_low = 0) or invalidate_cache then
    begin
      max := pattern(0);
      min := max;
      for i := 0 to pattern_x_range * pattern_x_div do
      begin
        x := i / pattern_x_div;
        y := pattern(x);
        if y > max then max := y;
        if y < min then min := y;
      end;
      size_cache_low := min;
      size_cache_high := max;
    end;
    size[0] := size_cache_low;
    size[1] := size_cache_high;
  end;

  { scale the pattern into the price rectangle defined by
  by width and last_bar. It uses the pattern height and
  the actual price range to calculate y_0 and y_scale.
  You can then just use scale[0] + scale[1] * pattern(x)
  to determine the absolute prices.}
  procedure scalePatternY(var scale: TTuple; Width, last_bar: integer);
  var
    pattern_size: TTuple; // 0:low 1:high
  begin
    calcPriceRange(Width, last_bar);
    patternSize(pattern_size, False);
    scale[1] := (price_high - price_low) / (pattern_size[1] - pattern_size[0]);
    if Width > 0 then scale[0] := price_low - pattern_size[0] * scale[1];
    if Width < 0 then scale[0] := price_low + pattern_size[1] * scale[1];
  end;

  { Calculate the error value for a given price range. The pattern
  will be scaled into the enclosing rectangle and the error between
  pattern and actual close prices will be calculated.}
  function error(Width, last_bar: integer): double;
  var
    i: integer;
    err: double;
    scale: TTuple;
    sign: integer;
    end_i: integer;
    fact_y: double;
    fact_x: double;

  begin
    scalePatternY(scale, Width, last_bar);
    err := 0;
    sign := 1;

    if Width < 0 then
    begin
      Width := -Width;
      sign := -1;
    end;

    fact_y := sign * scale[1];
    fact_x := pattern_x_range / Width;
    end_i := Bars - last_bar - 1;
    for i := 0 to Width do
    begin
      err += (abs(scale[0] + fact_y * pattern(fact_x * i) -
        history^[end_i - i].Close));
    end;
    error := err / (Width * (price_high - price_low) * history^[end_i].Close);
  end;

  { Loop through all widths and find the one with the smallest error
  range is the test range, all pattern widths up to range will be tested.
  The best fitted width is returned and it's corresponding error value will
  be written to a global variable. It can be queried with getLastError()}
  function findBest(range: integer): integer;
  var
    i: integer;
    e: double;
    e_min: double;
    r_best: integer;
  begin
    e_min := error(scan_min_width, 0);
    r_best := scan_min_width;

    for i := scan_min_width to range do
    begin
      e := error(i, 0);
      if e < e_min then
      begin
        e_min := e;
        r_best := i;
      end;

      e := error(-i, 0);
      if e < e_min then
      begin
        e_min := e;
        r_best := -i;
      end;
    end;

    last_error := e_min;
    findBest := r_best;
  end;

  { The error value of the best fitting pattern determined
  by the last call to findBest()}
  function lastError(): double; stdcall;
  begin
    lastError := last_error;
  end;

  { Set the Pointer to the MT4 rates array }
  procedure setRatesArray(ARates: PHistory; ABars: integer);
  begin
    history := ARates;
    Bars := ABars;
  end;

  { Set the parameters for the pattern function.}
  procedure setParams(p1, p2, p3: double);
  begin
    param1 := p1;
    param2 := p2;
    param3 := p3;
  end;

exports
  setRatesArray,
  setParams,
  pattern,
  calcPriceRange,
  scalePatternY,
  findBest,
  lastError;

begin
  size_cache_high := 0;
  size_cache_low := 0;
end.

